/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id: MetadataProperty.java,v d60681e7fc1d 2009/09/17 23:59:23 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.metadata.persistence;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import it.tidalwave.metadata.persistence.spi.BeanAccessor;
import it.tidalwave.metadata.persistence.spi.UnknownPropertyException;

/*******************************************************************************
 * 
 * This class describes a single property of a metadata item. 
 * 
 * @author  Fabrizio Giudici
 * @version $Id: MetadataProperty.java,v d60681e7fc1d 2009/09/17 23:59:23 fabrizio $
 *
 ******************************************************************************/
public class MetadataProperty<T> extends MetadataComposite
  {
    private final Map<String, PropertyAndValue<T>> cache = new HashMap<String, PropertyAndValue<T>>();
    
    /***************************************************************************
     *
     * Creates a new instance given a parent and a name.
     * 
     * @param  parent        the parent
     * @param  propertyName  the name 
     * 
     **************************************************************************/
    public MetadataProperty (@Nonnull final MetadataComposite parent,
                             @Nonnull final String propertyName) 
      {
        super(parent, propertyName);
      }

    /***************************************************************************
     *
     * Returns the property name.
     * 
     * @return   the name of the property
     * 
     **************************************************************************/
    @CheckForNull
    public String getPropertyName() 
      {
        return propertyName;
      }

    /***************************************************************************
     *
     * Returns the property type.
     *
     * @return   the type of the property
     *
     **************************************************************************/
    @Nonnull
    public Class<T> getType()
      throws UnknownPropertyException
      {
        final BeanAccessor<?> beanAccessor = persistenceSpi.findAccessorByClass(getItemClass());
        return (Class<T>)beanAccessor.getDefaultPropertyType(propertyName);
      }

    /***************************************************************************
     *
     * Finds or creates a new <code>PropertyAndValue</code> instance for the given
     * value. This method guarantees that always the same instance is returned 
     * for a given value; this saves time in case the method 
     * {@link PropertyAndValue#getDataObjectCount()} is called multiple times 
     * since it won't be recomputed for multiple instances, and furthermore it 
     * makes it possible for the Persistence Facility to constantly update the 
     * count when new data is added or removed.
     * 
     * @param  value     the value
     * 
     **************************************************************************/
    @Nonnull
    public synchronized PropertyAndValue<T> findOrCreatePropertyAndValue (@CheckForNull final T value)
      {
        // Use a String for the key so you can handle null and 
        // don't worry about value's equals() and hashCode()
        final String key = (value == null) ? "null" : value.toString();     
        PropertyAndValue<T> pav = cache.get(key);
        
        if (pav == null)
          {
            pav = new PropertyAndValue<T>(this, value);
            cache.put(key, pav);
          }
        
        return pav; 
      }
    
    /***************************************************************************
     *
     * Returns the list of all the possible values for this property. This 
     * method dinamically queries the persistent facility and always produces
     * an updated list.
     * 
     * @return   the values for this property
     * 
     **************************************************************************/
    @Nonnull
    public List<T> getValues()
      {
        return persistenceSpi.findValues(this);
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     * 
     **************************************************************************/
    @Override
    @Nonnull
    public final String toString()
      {
        return String.format("MetadataProperty[%s]", propertyName);
      }
  }  
